@@ -1,31 +1,32 @@
 #' Evaluation of Audit Samples using Confidence / Credible Bounds
 #'
-#' @description This function takes a sample data frame or summary statistics about an evaluated audit sample and calculates a confidence bound according to a specified method. The returned object is of class \code{jfaEvaluation} and can be used with associated \code{print()} and \code{plot()} methods.
+#' @description This function takes a data frame (using \code{sample}, \code{bookValue}, and \code{auditValues}) or summary statistics (using \code{nSumstats} and \code{kSumstats}) and evaluates the audit sample according to the specified method. The returned object is of class \code{jfaEvaluation} and can be used with associated \code{print()} and \code{plot()} methods.
 #'
 #' @usage evaluation(sample = NULL, bookValues = NULL, auditValues = NULL, 
-#'            confidence = 0.95, nSumstats = NULL, kSumstats = NULL,
+#'            counts = NULL, confidence = 0.95, nSumstats = NULL, kSumstats = NULL,
 #'            method = "binomial", materiality = NULL, N = NULL, 
 #'            prior = FALSE, nPrior = 0, kPrior = 0, 
 #'            rohrbachDelta = 2.7, momentPoptype = "accounts",
 #'            populationBookValue = NULL, minPrecision = NULL,
 #'            csA = 1, csB = 3, csMu = 0.5) 
 #'
-#' @param sample        a data frame containing at least a column of book values and a column of audit (true) values.
-#' @param bookValues    the column name for the book values in the sample.
-#' @param auditValues   the column name for the audit (true) values in the sample.
-#' @param confidence    the required confidence level for the bound.
-#' @param nSumstats     the number of observations in the sample. If specified, overrides the \code{sample}, \code{bookValues} and \code{auditValues} arguments and assumes that the data comes from summary statistics specified by \code{nSumstats} and \code{kSumstats}.
-#' @param kSumstats     the sum of the errors found in the sample. If specified, overrides the \code{sample}, \code{bookValues} and \code{auditValues} arguments and assumes that the data comes from summary statistics specified by \code{kSumstats} and \code{nSumstats}.
-#' @param method        can be either one of \code{poisson}, \code{binomial}, \code{hypergeometric}, \code{stringer}, \code{stringer-meikle}, \code{stringer-lta}, \code{stringer-pvz}, \code{rohrbach}, \code{moment}, \code{direct}, \code{difference}, \code{quotient}, or \code{regression}. 
-#' @param materiality   if specified, the function also returns the conclusion of the analysis with respect to the materiality. This value must be specified as a fraction of the total value of the population (a value between 0 and 1). The value is discarded when \code{direct}, \code{difference}, \code{quotient}, or \code{regression} method is chosen.
-#' @param N             the total population size.
-#' @param prior         whether to use a prior distribution when evaluating. Defaults to \code{FALSE} for frequentist evaluation. If \code{TRUE}, the prior distribution is updated by the specified likelihood. Chooses a conjugate gamma distribution for the Poisson likelihood, a conjugate beta distribution for the binomial likelihood, and a conjugate beta-binomial distribution for the hypergeometric likelihood.
-#' @param nPrior        the prior parameter \eqn{\alpha} (number of errors in the assumed prior sample).
-#' @param kPrior        the prior parameter \eqn{\beta} (total number of observations in the assumed prior sample).
-#' @param rohrbachDelta the value of \eqn{\Delta} in Rohrbach's augmented variance bound.
-#' @param momentPoptype can be either one of \code{accounts} or \code{inventory}. Options result in different methods for calculating the central moments, for more information see Dworin and Grimlund (1986).
-#' @param populationBookValue the total value of the audit population. Required when \code{method} is one of \code{direct}, \code{difference}, \code{quotient}, or \code{regression}.
-#' @param minPrecision  if specified, the function also returns the conclusion of the analysis with respect to the minimum precision. This value must be specified as a fraction of the total value of the population (a value between 0 and 1).
+#' @param sample        a data frame containing at least a column of Ist values and a column of Soll (true) values.
+#' @param bookValues    a character specifying the column name for the Ist values in the sample.
+#' @param auditValues   a character specifying the column name for the Soll values in the sample.
+#' @param counts        a integer vector of the number of times each transaction in the sample is to be evaluated (due to it being selected multiple times for the sample).
+#' @param confidence    the required confidence level for the bound. Default is 0.95 for 95\% confidence.
+#' @param nSumstats     an integer specifying the number of transactions in the sample. If specified, overrides the \code{sample}, \code{bookValues} and \code{auditValues} arguments and assumes that the data come from summary statistics specified by both \code{nSumstats} and \code{kSumstats}.
+#' @param kSumstats     a value specifying the sum of taints (proportional errors) found in the sample. If specified, overrides the \code{sample}, \code{bookValues} and \code{auditValues} arguments and assumes that the data come from summary statistics specified by both \code{kSumstats} and \code{nSumstats}.
+#' @param method        the method that is used to evaluate the sample. This can be either one of \code{poisson}, \code{binomial}, \code{hypergeometric}, \code{stringer}, \code{stringer-meikle}, \code{stringer-lta}, \code{stringer-pvz}, \code{rohrbach}, \code{moment}, \code{direct}, \code{difference}, \code{quotient}, or \code{regression}. 
+#' @param materiality   a value specifying the performance materiality as a fraction of the total value (or size) of the population (a value between 0 and 1). If specified, the function also returns the conclusion of the analysis with respect to the performance materiality. The value is discarded when \code{direct}, \code{difference}, \code{quotient}, or \code{regression} method is chosen.
+#' @param N             an integer specifying the total number of transactions in the population.
+#' @param prior         a logical indicating whether to use a prior distribution when evaluating. Defaults to \code{FALSE} for frequentist evaluation. If \code{TRUE}, the prior distribution is updated by the corresponding likelihood. Chooses a conjugate gamma distribution for the Poisson likelihood, a conjugate beta distribution for the binomial likelihood, and a conjugate beta-binomial distribution for the hypergeometric likelihood.
+#' @param nPrior        a value for the prior parameter \eqn{\beta} (number of transactions in the assumed prior sample).
+#' @param kPrior        a value for the prior parameter \eqn{\alpha} (total tainting in the assumed prior sample).
+#' @param rohrbachDelta a value specifying \eqn{\Delta} in Rohrbach's augmented variance bound (Rohrbach, 1993).
+#' @param momentPoptype a character specifying the type of population for the modified moment method (Dworin and Grimlund, 1986). Can be either one of \code{accounts} or \code{inventory}. Options result in different methods for calculating the central moments.
+#' @param populationBookValue a value specifying the total value of the transactions in the population. Required when \code{method} is one of \code{direct}, \code{difference}, \code{quotient}, or \code{regression}, but optional otherwise.
+#' @param minPrecision  a value specifying the required minimum precision. If specified, the function also returns the conclusion of the analysis with respect to the required minimum precision. This value must be specified as a fraction of the total value of the population (a value between 0 and 1).
 #' @param csA           if \code{method = "coxsnell"}, the \eqn{\alpha} parameter of the prior distribution on the mean taint. Default is set to 1, as recommended by Cox and Snell (1979).
 #' @param csB           if \code{method = "coxsnell"}, the \eqn{\beta} parameter of the prior distribution on the mean taint. Default is set to 3, as recommended by Cox and Snell (1979).
 #' @param csMu          if \code{method = "coxsnell"}, the mean of the prior distribution on the mean taint. Default is set to 0.5, as recommended by Cox and Snell (1979).
@@ -60,25 +61,25 @@
 #'
 #' @return An object of class \code{jfaEvaluation} containing:
 #' 
-#' \item{n}{the sample size.}
-#' \item{k}{an integer specifying the number of observed errors.}
-#' \item{t}{a number specifying the sum of observed taints.}
-#' \item{confidence}{the confidence level of the result.}
-#' \item{mle}{the most likely error in the population.}
-#' \item{precision}{the difference between the mle and the upper confidence bound.}
-#' \item{popBookvalue}{if specified as input, the total book value of the population.}
+#' \item{n}{an integer specifying the sample size used in the evaluation.}
+#' \item{k}{an integer specifying the number of transactions that contained an error.}
+#' \item{t}{a value specifying the sum of observed taints.}
+#' \item{confidence}{a value specifying the confidence level of the result.}
+#' \item{mle}{a value specifying the most likely error in the population as a proportion.}
+#' \item{precision}{a value specifying the difference between the mle and the upper confidence bound as a proportion.}
+#' \item{popBookvalue}{if specified as input, the total Ist value of the population.}
 #' \item{pointEstimate}{if method is one of \code{direct}, \code{difference}, \code{quotient}, or \code{regression}, the value of the point estimate.}
 #' \item{lowerBound}{if method is one of \code{direct}, \code{difference}, \code{quotient}, or \code{regression}, the value of the lower bound of the interval.}
 #' \item{upperBound}{if method is one of \code{direct}, \code{difference}, \code{quotient}, or \code{regression}, the value of the upper bound of the interval.}
 #' \item{confBound}{the upper confidence bound on the error percentage. }
 #' \item{method}{the evaluation method that was used.}
-#' \item{materiality}{the materiality.}
+#' \item{materiality}{if \code{materiality} is specified, the performance materiality used.}
 #' \item{conclusion}{if \code{materiality} is specified, the conclusion about whether to approve or not approve the population.}
-#' \item{N}{if specified as input, the population size.}
-#' \item{populationK}{the assumed total errors in the population. Used for inferences with \code{hypergeometric} method.}
-#' \item{prior}{a logical, indicating whether a prior was used in the analysis.}
-#' \item{nPrior}{if a prior is specified, the prior assumed sample size.}
-#' \item{kPrior}{if a prior is specified, the prior assumed sample errors.}
+#' \item{N}{if \code{N} is specified, the population size that is used.}
+#' \item{populationK}{the assumed total errors in the population. Used in inferences with \code{hypergeometric} method.}
+#' \item{prior}{a logical indicating whether a prior was used in the analysis.}
+#' \item{nPrior}{if a prior is specified, the assumed prior sample size.}
+#' \item{kPrior}{if a prior is specified, the assumed prior total taint.}
 #' \item{multiplicationFactor}{if \code{method = "coxsnell"}, the multiplication factor for the \emph{F}-distribution.}
 #' \item{df1}{if \code{method = "coxsnell"}, the df1 for the \emph{F}-distribution.}
 #' \item{df2}{if \code{method = "coxsnell"}, the df2 for the \emph{F}-distribution.}
@@ -130,10 +131,10 @@
 #' # Conclusion:               Approve population
 #' # ------------------------------------------------------------
 #'
-#' # Evaluate the raw sample using the stringer bound:
+#' # Evaluate the raw sample using the stringer bound and the sample counts:
 #' 
 #' e2 <- evaluation(sample = s1_sample, bookValues = "bookValue", auditValues = "trueValue", 
-#'                  method = "stringer", materiality = 0.05)
+#'                  method = "stringer", materiality = 0.05, counts = s1_sample$counts)
 #' print(e2)
 #' 
 #' # ------------------------------------------------------------ 
@@ -146,23 +147,23 @@
 #' # Minium precision:         100% 
 #' # Sample size:              100 
 #' # Sample errors:            1 
-#' # Sum of taints:            0.587 
+#' # Sum of taints:            0.644 
 #' # Method:                   stringer 
 #' # ------------------------------------------------------------
 #' # Output:
 #' #
-#' # Most likely error:        0.587% 
-#' # Upper bound:              3.952% 
-#' # Precision:                3.365% 
+#' # Most likely error:        0.644% 
+#' # Upper bound:              4.049% 
+#' # Precision:                3.405% 
 #' # Conclusion:               Approve population 
-#' # ------------------------------------------------------------
+#' # ------------------------------------------------------------  
 #' 
 #' @keywords evaluation confidence bound audit
 #'
 #' @export 
 
 evaluation <- function(sample = NULL, bookValues = NULL, auditValues = NULL, 
-                       confidence = 0.95, nSumstats = NULL, kSumstats = NULL,
+                       counts = NULL, confidence = 0.95, nSumstats = NULL, kSumstats = NULL,
                        method = "binomial", materiality = NULL, N = NULL, 
                        prior = FALSE, nPrior = 0, kPrior = 0, 
                        rohrbachDelta = 2.7, momentPoptype = "accounts",
@@ -183,6 +184,13 @@ evaluation <- function(sample = NULL, bookValues = NULL, auditValues = NULL,
   if(!(method %in% c("poisson", "binomial", "hypergeometric", "stringer", "stringer-meikle", "stringer-lta", "stringer-pvz",
                      "rohrbach", "moment", "coxsnell", "direct", "difference", "quotient", "regression")) || length(method) != 1)
     stop("Specify a valid method for the confidence bound")
+
+  if(((class(prior) == "logical" && prior == TRUE) || class(prior) == "jfaPrior") && method %in% c("stringer", "stringer-meikle", "stringer-lta", "stringer-pvz",
+                     "rohrbach", "moment", "direct", "difference", "quotient", "regression"))
+    stop("To use a prior distribution, use either the poisson, binomial, or hypergeometric method.")  
+
+  if(!is.null(counts) && any(counts < 1))
+    stop("Your counts must all be >= 1.")          
   
   if(!is.null(nSumstats) || !is.null(kSumstats)){
      if(is.null(nSumstats) || is.null(kSumstats))
@@ -207,11 +215,18 @@ evaluation <- function(sample = NULL, bookValues = NULL, auditValues = NULL,
     
     sample <- stats::na.omit(sample)
     n <- nrow(sample)
+    if(!is.null(counts))
+      n <- sum(counts)
     bv <- sample[, bookValues]
     av <- sample[, auditValues]
     taints <- (bv - av) / bv
     k <- length(which(taints != 0))
+
+    if(!is.null(counts))
+      taints <- taints * counts
+    
     t <- sum(taints)
+    
   }
   
   if(!is.null(materiality)){
